<?php

class nc_payment_system_tinkoff extends nc_payment_system
{
    const ERROR_MRCHLOGIN_IS_NOT_VALID = NETCAT_MODULE_PAYMENT_TINKOFF_ERROR_MRCHLOGIN_IS_NOT_VALID;
    const ERROR_INVID_IS_NOT_VALID = NETCAT_MODULE_PAYMENT_ERROR_INVID_IS_NOT_VALID;
    const ERROR_INVDESC_ID_IS_LONG = NETCAT_MODULE_PAYMENT_ERROR_INVDESC_ID_IS_LONG;
    const ERROR_PRIVATE_SECURITY_IS_NOT_VALID = NETCAT_MODULE_PAYMENT_TINKOFF_ERROR_TOKEN_IS_NOT_VALID;

    protected $automatic = TRUE;

    // принимаемые валюты
    protected $accepted_currencies = array('RUB', 'RUR');

    // параметры сайта в платежной системе
    protected $settings = array(
        'TerminalKey' => null,
        'Password' => null,
        'EnabledTaxation' => 0,
        'PaymentMethod' => null,
        'PaymentObject' => null,
        'Taxation' => null,
        'FFD'      => null, 
        'DeliveryTax' => 'none',
        'PaymentLanguage' => 'ru',
    );

    // передаваемые параметры
    protected $request_parameters = array();

    // получаемые параметры
    protected $callback_response = array(
        'InvId' => null,
        'OutSum' => null,
    );

    static public $tinkoff_vats = [
        'none' => 'none',
        'vat0' => 'vat0',
        'vat10' => 'vat10',
        'vat18' => 'vat18',
        'vat20' => 'vat20'
    ];


    public function execute_payment_request(nc_payment_invoice $invoice)
    {
        $amount = round($invoice->get_amount() * 100);
        $signature_values = array(
            $this->get_setting('TerminalKey'),
            $amount,
            $invoice->get_id(),
            $this->get_setting('Password'),
        );
        $signature = md5(implode(":", $signature_values));

        $requestData = array(
            'OrderId' => $invoice->get_id() ?: '',
            'Amount' => round($invoice->get_amount() * 100) ?: round('' * 100),
            'TerminalKey' => $this->get_setting('TerminalKey') ?: '',
            'Password' => $this->get_setting('Password') ?: '',
            'DATA' => array(
                'Email' => $invoice->get('customer_email'),
                'Connection_type' => 'netcat_1.0'
            ),
        );

        if ($this->get_setting('EnabledTaxation')) {
            $requestData['Receipt'] = $this->get_receipt($invoice);
        }

        if ($this->get_setting('PaymentLanguage') == 'en') {
            $requestData['Language'] = 'en';
        }

        $token = '';
        ksort($requestData);

        foreach ($requestData as $item) {
            if (!is_array($item)) {
                $token .= $item;
            }
        }

        $token = hash('sha256', $token);
        $requestData['Token'] = $token;
        unset($requestData['Password']);
        $args = json_encode($requestData);
     
 
        $url = 'https://securepay.tinkoff.ru/v2/' ?: '';

        if ($url[strlen($url) - 1] !== '/') {
            $url .= '/';
        }

        $url .= 'Init';

        if ($curl = curl_init()) {
            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $args);
            curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
            ));
            $out = curl_exec($curl);
            //  Раскомментируй строку ниже, если хочешь записать лог. После просмотра лога, обязательно удали созданный файл (tinkoff.log) и закомментируй строку.
            //  $this->logs($requestParams, $request, 'tinkoff.log');

            $json = json_decode($out);
            curl_close($curl);

            if ($json->PaymentURL) {
                header('Location: ' . $json->PaymentURL);
            }

            echo $json ? ($json->Message . ' ' . $json->Details) : $out;

        } else {
            throw new HttpException(
                'Can not create connection to ' . $url . ' with args '
                . $args, 404
            );
        }

        exit;
        ob_end_clean();
    }

    function logs($requestData, $args, $file)
    {
        $log = '[' . date('D M d H:i:s Y', time()) . '] ';
        $log .= json_encode($requestData, JSON_UNESCAPED_UNICODE);
        $log .= "\n";
        file_put_contents(dirname(__FILE__) . "\\" . $file, $log, FILE_APPEND);

        $log = '[' . date('D M d H:i:s Y', time()) . '] ';
        $log .= $args;
        $log .= "\n";
        file_put_contents(dirname(__FILE__) . "\\" . $file, $log, FILE_APPEND);
    }

    public function get_receipt($invoice)
    {
        $receiptItems = array();
        $sellReceipt = $invoice->get_sell_receipt();
        
        //добавим данные о товарах
    foreach ($sellReceipt->get_items() as $key => $item) {
        if ($item['source_item_id']) {
            if($this->get_setting('FFD') == '1.2') {
                $receiptItems[] = array(
                    'Name' => mb_substr($item['name'],0,64),
                    'Price' => round((float)$item->get('item_price') * 100),
                    'Quantity' => $item->get('qty'),
                    'Amount' => round($item->get('qty') * (float)$item->get('item_price') * 100),
                    'Tax' => nc_array_value(self::$tinkoff_vats, 'vat' . $item->get('vat_rate'), self::$tinkoff_vats['none']),
                    'PaymentMethod' => $this->get_setting('PaymentMethod'),
                    'PaymentObject' => $this->get_setting('PaymentObject'),
                    'MeasurementUnit' => 'pc' 
                );
            } else {
                $receiptItems[] = array(
                    'Name' => mb_substr($item['name'],0,64),
                    'Price' => round((float)$item->get('item_price') * 100),
                    'Quantity' => $item->get('qty'),
                    'Amount' => round($item->get('qty') * (float)$item->get('item_price') * 100),
                    'Tax' => nc_array_value(self::$tinkoff_vats, 'vat' . $item->get('vat_rate'), self::$tinkoff_vats['none']),    
                    'PaymentMethod' => $this->get_setting('PaymentMethod'),
                    'PaymentObject' => $this->get_setting('PaymentObject'),     
                );
            }

        }
      
    }

        $netshop = nc_netshop::get_instance();
        $order = $netshop->load_order($invoice->get('order_id'));

        //доставка
        if (isset($order['DeliveryCost']) && $order['DeliveryCost']) {
            $price = round((float)$order['DeliveryCost'] * 100);

            if (self::$tinkoff_vats[$this->get_setting('DeliveryTax')]) {
                $deliveryTax = self::$tinkoff_vats[$this->get_setting('DeliveryTax')];
            } else {
                $deliveryTax = self::$tinkoff_vats['none'];
            }
            if($this->get_setting('FFD') == '1.2') {
                $receiptItems[] = array(
                    'Name' => mb_substr($order->get_delivery_method()['name'],0,64),
                    'Price' => $price,
                    'Quantity' => 1,
                    'Amount' => $price,
                    'Tax' => $deliveryTax,
                    'PaymentMethod' => $this->get_setting('PaymentMethod'),
                    'PaymentObject' => 'service',
                    'MeasurementUnit' => 'pc'
                );
            } else {
                $receiptItems[] = array(
                    'Name' => mb_substr($order->get_delivery_method()['name'],0,64),
                    'Price' => $price,
                    'Quantity' => 1,
                    'Amount' => $price,
                    'Tax' => $deliveryTax,
                    'PaymentMethod' => $this->get_setting('PaymentMethod'),
                    'PaymentObject' => 'service',
                );
        }
    }

        //дополнительные сборы оплаты
        if (isset($order['PaymentCost']) && $order['PaymentCost']) {
            $price = round((float)$order['PaymentCost'] * 100);
            if($this->get_setting('FFD') == '1.2') {    
                $receiptItems[] = array(
                    'Name' => 'Комиссия за перечисление денежных средств',
                    'Price' => $price,
                    'Quantity' => 1,
                    'Amount' => $price,
                    'Tax' => nc_array_value(self::$tinkoff_vats, $this->get_setting('PaymentTax'), self::$tinkoff_vats['none']),
                    'PaymentMethod' => $this->get_setting('PaymentMethod'),
                    'PaymentObject' => $this->get_setting('PaymentObject'),     
                    'MeasurementUnit' => 'pc'
                );
            } else {
                $receiptItems[] = array(
                    'Name' => 'Комиссия за перечисление денежных средств',
                    'Price' => $price,
                    'Quantity' => 1,
                    'Amount' => $price,
                    'Tax' => nc_array_value(self::$tinkoff_vats, $this->get_setting('PaymentTax'), self::$tinkoff_vats['none']),
                    'PaymentMethod' => $this->get_setting('PaymentMethod'),
                    'PaymentObject' => $this->get_setting('PaymentObject'),
                );
            }
        }
       
        return array(
            'Email' => $invoice->get('customer_email'),
            'Taxation' => $this->get_setting('Taxation'),
            'Items' => $receiptItems,
            'FfdVersion' => $this->get_setting('FFD') == '1.2' ? '1.2' : '1.05'
        );
    }

    /**
     * @param nc_payment_invoice $invoice
     */
    public function on_response(nc_payment_invoice $invoice = null)
    {
        $netshop = nc_netshop::get_instance();
        $order = $netshop->load_order($invoice['order_id']);

        if ($_POST['Status'] == 'AUTHORIZED' && $order['Status'] == 3) {
            die('OK');
        }

        switch ($_POST['Status']) {
            case 'AUTHORIZED':
                $order_status = 1;
                break;
            case 'CONFIRMED':
                $order_status = 3;
                break;
            case 'CANCELED':
                $order_status = 2;
                break;
            case 'REJECTED':
                $order_status = 2;
                break;
            case 'REVERSED':
                $order_status = 2;
                break;
            case 'REFUNDED':
                $order_status = 2;
                break;
        }

        $order->set('Status', $order_status);
        $order->save();

        if ($order_status == 3) {
            $this->on_payment_success($invoice);
        }

        echo 'OK';
    }

    /**
     *
     */
    public function validate_payment_request_parameters()
    {
        return true;
    }

    /**
     * @param nc_payment_invoice $invoice
     */
    public function validate_payment_callback_response(nc_payment_invoice $invoice = null)
    {
        $_POST['Password'] = $this->get_setting('Password');
        ksort($_POST);
        $sorted = $_POST;
        $original_token = $sorted['Token'];
        unset($sorted['Token']);
        $values = implode('', array_values($sorted));
        $token = hash('sha256', $values);

        if ((int)$_POST["Amount"] !== (int)round($invoice["amount"] * 100)) {
            die("AMOUNTS DO NOT MATCH");
        }

        if (!$invoice) {
            die('ORDER NOT FOUND');
        }

        if ($token !== $original_token) {
            die('TOKEN INCORRECT');
        }
    }

    public function load_invoice_on_callback()
    {
        return $this->load_invoice($this->get_response_value('OrderId'));
    }
}

